/*
*******************************************************************************
* $Header: $
*
*  Copyright (c) 2018-2018 VIVOTEK Inc. All rights reserved.
*
*  +-----------------------------------------------------------------+
*  | THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY ONLY BE USED |
*  | AND COPIED IN ACCORDANCE WITH THE TERMS AND CONDITIONS OF SUCH  |
*  | A LICENSE AND WITH THE INCLUSION OF THE THIS COPY RIGHT NOTICE. |
*  | THIS SOFTWARE OR ANY OTHER COPIES OF THIS SOFTWARE MAY NOT BE   |
*  | PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY OTHER PERSON. THE   |
*  | OWNERSHIP AND TITLE OF THIS SOFTWARE IS NOT TRANSFERRED.        |
*  |                                                                 |
*  | THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT   |
*  | ANY PRIOR NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY |
*  | VIVOTEK INC.                                                    |
*  +-----------------------------------------------------------------+
*
* $History: $
*/

/*!
 *******************************************************************************
 * Copyright 2000-2018 VIVOTEK, Inc. All rights reserved.
 *
 * \file
 * PT5111.c
 *
 * \brief
 * Source file of motor control driver.
 *
 * \date
 * 2018/03/31
 *
 * \author
 * Tim Kuo <Tim.Kuo@vivotek.com>
 *
 *******************************************************************************
 */

#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/fcntl.h>
#include <linux/mm.h>
#include <linux/proc_fs.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/io.h>


#include <linux/miscdevice.h>
#include <linux/delay.h>

#include <linux/proc_fs.h>
#include <linux/poll.h>

#include <asm/bitops.h>
#include <asm/uaccess.h>
#include <asm/irq.h>

#include <linux/moduleparam.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>

#include "motor_cfg.h"

//-------------------------------------------------------------------------//
//channel
//bit 14~15
#define GROUP_A						0x0		// channel 1 and channel 2
#define GROUP_B						0x4000	// channel 3 and channel 4
#define GROUP_C						0x8000	// channel 5 and channel 7
//-------------------------------------------------------------------------//
//register
//bit 10~13
#define REG_PULSE_RATE_0			0xC00
#define REG_PULSE_RATE_RANGE		0x1000

#define REG_PULSE1					0x400
#define REG_PULSE2					0x800

#define REG_PRE_EXCITATION			0x2800
#define REG_POST_EXCITATION			0x2C00

#define REG_OPERATION_CTRL			0x3C00

#define REG_PWM_MODE				0x3000	// Only for Channel 5 and 7, Channels 5 and 7 are full-swing PWM
#define REG_PWM_DUTY_CH5			0x3400	// Only for Channel 5
//-------------------------------------------------------------------------//
//setting
//bit 9
#define SET_FORWARD					0x0
#define SET_BACKWARD				0x200

#define SET_OPERATION_DISABLE		0x0
#define SET_OPERATION_ENABLE		0x200

//bit 8
#define SET_PWM_STOP_MODE_OFF		0x0
#define SET_PWM_STOP_MODE_BRAKE		0x100

#define SET_OPERATION_STATE_RELEASE	0x0
#define SET_OPERATION_STATE_HOLD	0x100

//bit 8~9

#define	SET_DUTY_0					0x0
#define	SET_DUTY_100				0x300

//bit 7
#define SET_EXCITATION_DISABLE		0x0
#define SET_EXCITATION_ENABLE		0x80

#define SET_PWM_CTRL_MODE_BRAKE		0x0
#define SET_PWM_CTRL_MODE_REVERSE	0x80

//bit 6
#define SET_PWM_CARRIER_FREQ_D32	0x0
#define SET_PWM_CARRIER_FREQ_D48	0x40

#define SET_CH7_OPERATION_DISABLE	0x0
#define SET_CH7_OPERATION_ENABLE	0x40

//bit 6~8
#define SET_MICOROSTEP				0x0
#define SET_1_2PHASE_100			0x40	// 100%
#define SET_1_2PHASE_70				0x80	//  70%
#define SET_2_2PHASE				0xC0

#define SET_FULL_SWING_PWM			0x100	// Only for Channel 5 and 7, Channels 5 and 7 are full-swing PWM

//bit 6~9
#define SET_RANGE3_2				0x180	// 3.2 us	
#define SET_RANGE1_6				0x1C0	// 1.6 us	
#define SET_RANGE0_8				0x200	// 0.8 us	
#define SET_RANGE0_4				0x240	// 0.4 us	

//bit 6~9
#define SET_EXCITATION_TIME			0x20	// (4096*[2]/OSCIN[1Mhz]) 8.192ms

//bit 0~9
#define SET_PULSE_RATE_NUM_500		0x1F4	// 500
#define SET_PULSE_RATE_NUM_625		0x271	// 625
#define SET_PULSE_RATE_NUM_1000		0x3E8	// 1000

//-------------------------------------------------------------------------//
#ifndef __iomem
#define __iomem
#endif

#if 0
#define  ssp_readw(addr,ret)			(ret =(*(volatile unsigned int *)(addr)))
#define  ssp_writew(addr,value)			((*(volatile unsigned int *)(addr)) = (value))
#define  HI_REG_READ(addr,ret)			(ret =(*(volatile unsigned int *)(addr)))
#define  HI_REG_WRITE(addr,value)		((*(volatile unsigned int *)(addr)) = (value))	

#define SSP_BASE	0x12124000      	//SPI4
#define SSP_SIZE	0x10000				//64KB
#define SSP_INT		41					//Interrupt No.

#define MISC_BASE	0x12030000      	//MISC
#define MISC_SIZE	0x10000				//64KB
//-------------------------------------------------------------------------//
// SSP BASE
void __iomem *reg_ssp_base_va;
#define IO_ADDRESS_VERIFY(x) (reg_ssp_base_va + ((x)-(SSP_BASE)))

/* SSP register definition .*/
#define SSP_CR0              IO_ADDRESS_VERIFY(SSP_BASE + 0x00)
#define SSP_CR1              IO_ADDRESS_VERIFY(SSP_BASE + 0x04)
#define SSP_DR               IO_ADDRESS_VERIFY(SSP_BASE + 0x08)
#define SSP_SR               IO_ADDRESS_VERIFY(SSP_BASE + 0x0C)
#define SSP_CPSR             IO_ADDRESS_VERIFY(SSP_BASE + 0x10)
#define SSP_IMSC             IO_ADDRESS_VERIFY(SSP_BASE + 0x14)
#define SSP_RIS              IO_ADDRESS_VERIFY(SSP_BASE + 0x18)
#define SSP_MIS              IO_ADDRESS_VERIFY(SSP_BASE + 0x1C)
#define SSP_ICR              IO_ADDRESS_VERIFY(SSP_BASE + 0x20)
#define SSP_DMACR            IO_ADDRESS_VERIFY(SSP_BASE + 0x24)
//-------------------------------------------------------------------------//
// MISC BASE
void __iomem *reg_misc_base_va;
#define MISC_IO_ADDRESS_VERIFY(x) (reg_misc_base_va + ((x)-(MISC_BASE)))

/* MISC register definition .*/
#define MISC_CTRL3			MISC_IO_ADDRESS_VERIFY(MISC_BASE + 0xC)		// spi cs reverse
#define MISC_CTRL23			MISC_IO_ADDRESS_VERIFY(MISC_BASE + 0x5C)	// spi4_cs_sel
//-------------------------------------------------------------------------//
#endif

void pt5111_init_cfg()
{
	unsigned char framemode = 0;
	unsigned char spo = 1;
	unsigned char sph = 1;
	unsigned char datawidth = 16;

	unsigned char scr = 24;
	unsigned char cpsdvsr = 8;

	spi_disable();
	hi_ssp_set_spi4_reverse();
	hi_ssp_set_frameform(framemode, spo, sph, datawidth);
	hi_ssp_set_serialclock(scr, cpsdvsr);
	hi_ssp_alt_mode_set(1);
	hi_ssp_enable();

	spi_write_a16byte(0xCFC0, 0);//1100 1111 1100 0000   ps
	spi_write_a16byte(0xCFC0, 1);//1100 1111 1100 0000   ps
	spi_write_a16byte(0xCFC0, 2);//1100 1111 1100 0000   ps
	spi_write_a16byte(0xCFC0, 3);//1100 1111 1100 0000   ps
}

unsigned int pt5111_SetChannelGroup(TMotorDev *ptDevice)
{
	switch(ptDevice->iMotorType)
	{
		case eZoom:
			return GROUP_A;
			break;
		case eFocus:
			return GROUP_B;
			break;
		default:
			DBG(emDebugMsg, "SetChannelGroup fail\n");
			return 0x0;
			break;
	}
}

int pt5111_SetDirection(TMotorDev *ptDevice)
{
	unsigned int SPI_cmd = pt5111_SetChannelGroup(ptDevice);

	SPI_cmd |= (ptDevice->bDirection ^ ptDevice->dwCtrlLogic) ? SET_FORWARD : SET_BACKWARD;
	//DBG(emDebugMsg, "ptDevice->dwCtrlPhase: %d, iMotorType: %d\n", ptDevice->dwCtrlPhase, ptDevice->iMotorType);

	switch(ptDevice->dwCtrlPhase)
	{
		case eMicoroStep:
			SPI_cmd |= SET_MICOROSTEP;
			break;
		case e1_2phase_100:
			SPI_cmd |= SET_1_2PHASE_100;
			break;
		case e1_2phase_70:
			SPI_cmd |= SET_1_2PHASE_70;
			break;
		case e2_2phase:
			SPI_cmd |= SET_2_2PHASE;
			break;
		default:
			return S_FAIL;
			break;
	}
	
	spi_write_a16byte(SPI_cmd, ptDevice->dwDeviceSelect);
	return S_OK;
}

int pt5111_SetPPS(TMotorDev *ptDevice)
{
	unsigned int SPI_cmd = pt5111_SetChannelGroup(ptDevice);;

	switch(ptDevice->dwSpeed)
	{
		case 1000:
			// 1000PPS = 1000us
			// 2us * 500 = 1000us
			SPI_cmd |= REG_PULSE_RATE_RANGE | SET_RANGE0_4;
			spi_write_a16byte(SPI_cmd, ptDevice->dwDeviceSelect);

			SPI_cmd = pt5111_SetChannelGroup(ptDevice);
			SPI_cmd |= REG_PULSE_RATE_0 | SET_PULSE_RATE_NUM_500;
			spi_write_a16byte(SPI_cmd, ptDevice->dwDeviceSelect);
			break;

		case 800:
			// 800PPS = 1250us
			// 2us * 625 = 1250us
			SPI_cmd |= REG_PULSE_RATE_RANGE | SET_RANGE0_4;
			spi_write_a16byte(SPI_cmd, ptDevice->dwDeviceSelect);

			SPI_cmd = pt5111_SetChannelGroup(ptDevice);
			SPI_cmd |= REG_PULSE_RATE_0 | SET_PULSE_RATE_NUM_625;
			spi_write_a16byte(SPI_cmd, ptDevice->dwDeviceSelect);
			break;

		case 500:
			// 500PPS = 2000us
			// 2us * 625 = 2000us
			SPI_cmd |= REG_PULSE_RATE_RANGE | SET_RANGE0_4;
			spi_write_a16byte(SPI_cmd, ptDevice->dwDeviceSelect);

			SPI_cmd = pt5111_SetChannelGroup(ptDevice);
			SPI_cmd |= REG_PULSE_RATE_0 | SET_PULSE_RATE_NUM_1000;
			spi_write_a16byte(SPI_cmd, ptDevice->dwDeviceSelect);
			break;
		
		case 100:
		default:
			// 100PPS = 10000us
			// 16us * 625 = 10000us
			SPI_cmd |= REG_PULSE_RATE_RANGE | SET_RANGE3_2;
			spi_write_a16byte(SPI_cmd, ptDevice->dwDeviceSelect);

			SPI_cmd = pt5111_SetChannelGroup(ptDevice);
			SPI_cmd |= REG_PULSE_RATE_0 | SET_PULSE_RATE_NUM_625;
			spi_write_a16byte(SPI_cmd, ptDevice->dwDeviceSelect);
			break;
	}
	
	return S_OK;
}

int pt5111_SetStep(TMotorDev *ptDevice)
{
	unsigned int low_pulse = 0, high_pulse = 0;
	unsigned int SPI_cmd = pt5111_SetChannelGroup(ptDevice);

	low_pulse = (ptDevice->dwSpareSteps % 1024);
	high_pulse = (ptDevice->dwSpareSteps / 1024);

	//DBG(emDebugMsg, "low_pulse: %d, high_pulse: %d\n", low_pulse, high_pulse);
	SPI_cmd |= REG_PULSE1 | low_pulse;
	spi_write_a16byte(SPI_cmd, ptDevice->dwDeviceSelect);

	SPI_cmd = pt5111_SetChannelGroup(ptDevice);
	SPI_cmd |= REG_PULSE2 | high_pulse;
	spi_write_a16byte(SPI_cmd, ptDevice->dwDeviceSelect);

	return S_OK;
}

int pt5111_SetPreExcitation(TMotorDev *ptDevice)
{
	unsigned int SPI_cmd = pt5111_SetChannelGroup(ptDevice);

	//DBG(emDebugMsg, "low_pulse: %d, high_pulse: %d\n", low_pulse, high_pulse);
	SPI_cmd |= REG_PRE_EXCITATION | SET_EXCITATION_TIME;
	spi_write_a16byte(SPI_cmd, ptDevice->dwDeviceSelect);

	return S_OK;
}

int pt5111_SetPostExcitation(TMotorDev *ptDevice)
{
	unsigned int SPI_cmd = pt5111_SetChannelGroup(ptDevice);

	//DBG(emDebugMsg, "low_pulse: %d, high_pulse: %d\n", low_pulse, high_pulse);
	SPI_cmd |= REG_POST_EXCITATION | SET_EXCITATION_TIME;
	spi_write_a16byte(SPI_cmd, ptDevice->dwDeviceSelect);

	return S_OK;
}

int pt5111_SetOperation(TMotorDev *ptDevice)
{
	unsigned int SPI_cmd = pt5111_SetChannelGroup(ptDevice);

	//SPI_cmd |= REG_OPERATION_CTRL | SET_OPERATION_ENABLE | SET_EXCITATION_ENABLE;
	SPI_cmd |= REG_OPERATION_CTRL | SET_OPERATION_ENABLE | SET_EXCITATION_DISABLE;
	spi_write_a16byte(SPI_cmd, ptDevice->dwDeviceSelect);

	return S_OK;
}

INT pt5111_start(TMotorDev *ptDevice)
{
	//DBG(emDebugMsg, "pt5111_start\n");
	//pt5111_init_cfg();
	hi_ssp_enable();

	pt5111_SetDirection(ptDevice);

	pt5111_SetPPS(ptDevice);

	pt5111_SetStep(ptDevice);

	pt5111_SetPreExcitation(ptDevice);

	pt5111_SetPostExcitation(ptDevice);

	pt5111_SetOperation(ptDevice);

	return S_OK;
}

INT motor_register_clean(TMotorDev *ptDevice)
{
	return S_OK;
}

INT motor_register_free(TMotorDev *ptDevice)
{
	return S_OK;
}

INT hi_ssp_init(void)
{
#if 0
	reg_ssp_base_va = ioremap_nocache((unsigned int)SSP_BASE, (unsigned int)SSP_SIZE);
	if (!reg_ssp_base_va)
	{
		printk("Kernel: ioremap ssp base failed!\n");
	    return -ENOMEM;
	}

	reg_misc_base_va = ioremap_nocache((unsigned int)MISC_BASE, (unsigned int)MISC_SIZE);
	if (!reg_misc_base_va)
	{
		printk("Kernel: ioremap ssp base failed!\n");
	    return -ENOMEM;
	}
#endif
	//DBG(emDebugMsg, "hi_ssp_init ok!\n");
    return S_OK;
}

void hi_ssp_exit(void)
{
#if 0
    hi_ssp_disable();

    iounmap((void*)reg_ssp_base_va);
	iounmap((void*)reg_misc_base_va);
#endif
}

