/*
*******************************************************************************
* $Header: $
*
*  Copyright (c) 2000-2018 VIVOTEK Inc. All rights reserved.
*
*  +-----------------------------------------------------------------+
*  | THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY ONLY BE USED |
*  | AND COPIED IN ACCORDANCE WITH THE TERMS AND CONDITIONS OF SUCH  |
*  | A LICENSE AND WITH THE INCLUSION OF THE THIS COPY RIGHT NOTICE. |
*  | THIS SOFTWARE OR ANY OTHER COPIES OF THIS SOFTWARE MAY NOT BE   |
*  | PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY OTHER PERSON. THE   |
*  | OWNERSHIP AND TITLE OF THIS SOFTWARE IS NOT TRANSFERRED.        |
*  |                                                                 |
*  | THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT   |
*  | ANY PRIOR NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY |
*  | VIVOTEK INC.                                                    |
*  +-----------------------------------------------------------------+
*
* $History: $
*/

/*!
*******************************************************************************
* Copyright 2000-2018 VIVOTEK, Inc. All rights reserved.
*
* \file
* motor_api.c
*
* \brief
* Central useful implementations.
*
*
* \date
* 2018/03/31
*
* \author
* Spark Hsieh <spark.hsieh@vivotek.com>
*
*******************************************************************************
*/

#include "motor_cfg.h"
//extern char* MOTOR_PROFILE_PATH;
//const char* MOTOR_PROFILE_PATH = "/etc/conf.d/lensmotor.cfg";


/*!**************************************************************************
* Parsing all parameters from config file.
*****************************************************************************/
INT motor_parsing_params(TMotorDev *ptDevice, char* pFileBuf)
{
	TCtrlRegister* ptCtrlRegister;
	TCtrlRegister* ptHeaterCtrlReg;
	TCtrlRegister* ptPICtrlReg;
	TCtrlRegister* ptPIPowerReg;
	PCHAR pValueBuf;
	char *pcStrEnd;

	DBG(emDebugInfo, "Parsing params...\n");
	if ((pValueBuf = kmalloc(MAX_PARAM_BUFFER_SIZE, GFP_KERNEL)) == NULL)
	{
		DBG_ERR("Can't get memory in motor_parsing_params.\n");
		return S_FAIL;
	}

	if ((ptCtrlRegister = kmalloc(sizeof(TCtrlRegister), GFP_ATOMIC)) == NULL)		// GFP_ATOMIC won't sleep while content switch.
	{
		kfree(pValueBuf);
		DBG_ERR("Can't get memory in motor_parsing_params.\n");
		return S_FAIL;
	}

	if ((ptHeaterCtrlReg = kmalloc(sizeof(TCtrlRegister), GFP_ATOMIC)) == NULL)	// GFP_ATOMIC won't sleep while content switch.
	{
		kfree(pValueBuf);
		DBG_ERR("Can't get memory in motor_parsing_params.\n");
		return S_FAIL;
	}

	if ((ptPICtrlReg = kmalloc(sizeof(TCtrlRegister), GFP_ATOMIC)) == NULL)		// GFP_ATOMIC won't sleep while content switch.
	{
		kfree(pValueBuf);
		DBG_ERR("Can't get memory in motor_parsing_params.\n");
		return S_FAIL;
	}

	if ((ptPIPowerReg = kmalloc(sizeof(TCtrlRegister), GFP_ATOMIC)) == NULL)		// GFP_ATOMIC won't sleep while content switch.
	{
		kfree(pValueBuf);
		DBG_ERR("Can't get memory in motor_parsing_params.\n");
		return S_FAIL;
	}

	ptDevice->tCtrlRegister 	= (void*)ptCtrlRegister;
	ptDevice->tHeaterCtrlReg 	= (void*)ptHeaterCtrlReg;
	ptDevice->tPICtrlReg 		= (void*)ptPICtrlReg;
	ptDevice->tPIPowerReg		= (void*)ptPIPowerReg;

	do
	{
		if(motor_get_target("ProjectName", pValueBuf, MAX_NAME_LENGTH, pFileBuf))
		{
			memmove(ptDevice->szProjectName, pValueBuf, strlen(pValueBuf));
			ptDevice->szProjectName[strlen(pValueBuf)] = '\0';
			//DBG(emDebugInfo, "ProjectName=%s.\n", ptDevice->szProjectName);
		}

		if(motor_get_target("LensName", pValueBuf, MAX_NAME_LENGTH, pFileBuf))
		{
			memmove(ptDevice->szLensName, pValueBuf, strlen(pValueBuf));
			ptDevice->szLensName[strlen(pValueBuf)] = '\0';
			//DBG(emDebugInfo, "LensName=%s.\n", ptDevice->szLensName);
		}

		if(motor_get_target("CtrlIcName", pValueBuf, MAX_NAME_LENGTH, pFileBuf))
		{
			memmove(ptDevice->szCtrlIcName, pValueBuf, strlen(pValueBuf));
			ptDevice->szCtrlIcName[strlen(pValueBuf)] = '\0';
			//DBG(emDebugInfo, "CtrlIcName=%s.\n", ptDevice->szCtrlIcName);
		}

		if(motor_get_target("DeviceName", pValueBuf, MAX_NAME_LENGTH, pFileBuf))
		{
			motor_get_keyvalue(ptDevice->iDeviceId, pValueBuf);
			memmove(ptDevice->szDeviceName, pValueBuf, strlen(pValueBuf));
			ptDevice->szDeviceName[strlen(pValueBuf)] = '\0';

			if (strcmp(ptDevice->szDeviceName, "zoommotor") == 0)
			{
				ptDevice->iMotorType = eZoom;
			}
			else if(strcmp(ptDevice->szDeviceName, "focusmotor") == 0)
			{
				ptDevice->iMotorType = eFocus;
			}
			else if(strcmp(ptDevice->szDeviceName, "pirismotor") == 0)
			{
				ptDevice->iMotorType = ePIris;
			}

			//DBG(emDebugInfo, "DeviceName=%s.\n", ptDevice->szDeviceName);
		}

		if(motor_get_target("CtrlPhase", pValueBuf, MAX_PARAM_LENGTH, pFileBuf))
		{
			motor_get_keyvalue(ptDevice->iDeviceId, pValueBuf);
			simple_strtol(pValueBuf, &pcStrEnd, DEC);
			if ( (pcStrEnd != pValueBuf && *pcStrEnd == '\0') )
				ptDevice->dwCtrlPhase = simple_strtol(pValueBuf, &pcStrEnd, DEC);
			//DBG(emDebugInfo, "CtrlPhase=%u.\n", ptDevice->dwCtrlPhase);
		}

		if(motor_get_target("CtrlLogic", pValueBuf, MAX_PARAM_LENGTH, pFileBuf))
		{
			motor_get_keyvalue(ptDevice->iDeviceId, pValueBuf);
			simple_strtol(pValueBuf, &pcStrEnd, DEC);
			if ( (pcStrEnd != pValueBuf && *pcStrEnd == '\0') )
				ptDevice->dwCtrlLogic = simple_strtol(pValueBuf, &pcStrEnd, DEC);
			//DBG(emDebugInfo, "CtrlLogic=%u.\n", ptDevice->dwCtrlLogic);
		}

		if(motor_get_target("MaxSpeed", pValueBuf, MAX_PARAM_LENGTH, pFileBuf))
		{
			motor_get_keyvalue(ptDevice->iDeviceId, pValueBuf);
			simple_strtol(pValueBuf, &pcStrEnd, DEC);
			if ( (pcStrEnd != pValueBuf && *pcStrEnd == '\0') )
				ptDevice->dwSpeed = simple_strtol(pValueBuf, &pcStrEnd, DEC);
			//DBG(emDebugInfo, "MaxSpeed=%u.\n", ptDevice->dwSpeed);
		}
		else
		{
			ptDevice->dwSpeed = MOTOR_DEFAULT_SPEED;
			//DBG(emDebugInfo, "Default MaxSpeed=%u.\n", ptDevice->dwSpeed);
		}
		
		if(motor_get_target("PISupport", pValueBuf, MAX_PARAM_LENGTH, pFileBuf))
		{
			motor_get_keyvalue(ptDevice->iDeviceId, pValueBuf);
			simple_strtol(pValueBuf, &pcStrEnd, DEC);
			if ( (pcStrEnd != pValueBuf && *pcStrEnd == '\0') )
				ptDevice->dwPISupport = simple_strtol(pValueBuf, &pcStrEnd, DEC);
			//DBG(emDebugInfo, "dwPISupport=%u.\n", ptDevice->dwPISupport);
		}
		else
		{
			ptDevice->dwPISupport = 0;
			//DBG(emDebugInfo, "default dwPISupport=%u.\n", ptDevice->dwPISupport);
		}

		if(motor_get_target("PISwitch", pValueBuf, MAX_PARAM_LENGTH, pFileBuf))
		{
			motor_get_keyvalue(ptDevice->iDeviceId, pValueBuf);
			simple_strtol(pValueBuf, &pcStrEnd, DEC);
			if ( (pcStrEnd != pValueBuf && *pcStrEnd == '\0') )
				ptDevice->dwPISwitch = simple_strtol(pValueBuf, &pcStrEnd, DEC);
			//DBG(emDebugInfo, "dwPISwitch=%u.\n", ptDevice->dwPISwitch);
		}
		else
		{
			ptDevice->dwPISupport = 0;
			//DBG(emDebugInfo, "default dwPISupport=%u.\n", ptDevice->dwPISupport);
		}

		if(motor_get_target("PICtrlPortCH0", pValueBuf, MAX_PARAM_LENGTH, pFileBuf))
		{
			motor_get_keyvalue(ptDevice->iDeviceId, pValueBuf);
			simple_strtol(pValueBuf, &pcStrEnd, DEC);
			if ((pcStrEnd != pValueBuf && *pcStrEnd == '\0') )
					ptPICtrlReg->uiCtrlPort = ptPICtrlReg->uiCtrlPortInfo[0] = simple_strtol(pValueBuf, &pcStrEnd, DEC);
			//DBG(emDebugInfo, "PICtrlPortCH0=%u.\n", ptPICtrlReg->uiCtrlPortInfo[0]);
		}

		if(motor_get_target("PICtrlPortCH1", pValueBuf, MAX_PARAM_LENGTH, pFileBuf))
		{
			motor_get_keyvalue(ptDevice->iDeviceId, pValueBuf);
			simple_strtol(pValueBuf, &pcStrEnd, DEC);
			if ((pcStrEnd != pValueBuf && *pcStrEnd == '\0') )
					ptPICtrlReg->uiCtrlPortInfo[1] = simple_strtol(pValueBuf, &pcStrEnd, DEC);
			//DBG(emDebugInfo, "PICtrlPortCH1=%u.\n", ptPICtrlReg->uiCtrlPortInfo[1]);
		}

		if(motor_get_target("PICtrlPortCH2", pValueBuf, MAX_PARAM_LENGTH, pFileBuf))
		{
			motor_get_keyvalue(ptDevice->iDeviceId, pValueBuf);
			simple_strtol(pValueBuf, &pcStrEnd, DEC);
			if ((pcStrEnd != pValueBuf && *pcStrEnd == '\0') )
					ptPICtrlReg->uiCtrlPortInfo[2] = simple_strtol(pValueBuf, &pcStrEnd, DEC);
			//DBG(emDebugInfo, "PICtrlPortCH2=%u.\n", ptPICtrlReg->uiCtrlPortInfo[2]);
		}

		if(motor_get_target("PICtrlPortCH3", pValueBuf, MAX_PARAM_LENGTH, pFileBuf))
		{
			motor_get_keyvalue(ptDevice->iDeviceId, pValueBuf);
			simple_strtol(pValueBuf, &pcStrEnd, DEC);
			if ((pcStrEnd != pValueBuf && *pcStrEnd == '\0') )
					ptPICtrlReg->uiCtrlPortInfo[3] = simple_strtol(pValueBuf, &pcStrEnd, DEC);
			//DBG(emDebugInfo, "PICtrlPortCH3=%u.\n", ptPICtrlReg->uiCtrlPortInfo[3]);
		}

		if(motor_get_target("PIPowerPort", pValueBuf, MAX_PARAM_LENGTH, pFileBuf))
		{
			motor_get_keyvalue(ptDevice->iDeviceId, pValueBuf);
			simple_strtol(pValueBuf, &pcStrEnd, DEC);
			if ((pcStrEnd != pValueBuf && *pcStrEnd == '\0') )
					ptPIPowerReg->uiCtrlPort = simple_strtol(pValueBuf, &pcStrEnd, DEC);
			//DBG(emDebugInfo, "PIPowerPort=%u.\n", ptPIPowerReg->uiCtrlPort);
		}

		if(motor_get_target("TotalStep", pValueBuf, MAX_PARAM_LENGTH, pFileBuf))
		{
			motor_get_keyvalue(ptDevice->iDeviceId, pValueBuf);
			simple_strtol(pValueBuf, &pcStrEnd, DEC);
			if ( (pcStrEnd != pValueBuf && *pcStrEnd == '\0') )
				ptDevice->dwTotalStep = simple_strtol(pValueBuf, &pcStrEnd, DEC);
			//DBG(emDebugInfo, "dwTotalStep=%u.\n", ptDevice->dwTotalStep);
		}
		else
		{
			ptDevice->dwTotalStep = MAX_MOTOR_STEP;
			//DBG(emDebugInfo, "dwTotalStep=%u.\n", ptDevice->dwTotalStep);
		}
	}while(0);

	ptDevice->dwDeviceSelect = 0;

	kfree(pValueBuf);
	return	S_OK;
}

/*!**************************************************************************
* Read parameters from Config File.
*****************************************************************************/
INT motor_init_params(TMotorDev *ptDevice)
{
	extern PCHAR g_config;
	INT iReturn = S_FAIL;

	DBG(emDebugInfo, "Get into motor_init_params.\n");

	if(strcmp(g_config, ""))
	{
		DBG(emDebugInfo, "Get parameters while insmod.\n");
		motor_parsing_params(ptDevice, g_config);
		return S_OK;
	}
	else
	{
		DBG(emDebugInfo, "Input parameters while insmod.\n");
		return iReturn;
	}
}

/*!**************************************************************************
* Get the target value of Config File.
*****************************************************************************/
INT motor_get_target(PCHAR pTag, PCHAR pDest, INT iDestsize, PCHAR pSrc)
{
	extern PCHAR g_config;
	PCHAR pMemBuf, pTmpBuf1 = NULL, pTmpBuf2 = NULL;
	PCHAR pOffset = pSrc;
	PCHAR ptrStart, ptrEnd, ptrValue;
	INT  iLen;

	pMemBuf = kmalloc(MAX_PARAM_BUFFER_SIZE * 2, GFP_KERNEL);
	if (pMemBuf == NULL)
	{
		DBG_ERR("motor_get_target: no memory.\n");
		return FALSE;
	}
	memset(pMemBuf, 0, MAX_PARAM_BUFFER_SIZE * 2);

	pTmpBuf1 = pMemBuf;
	pTmpBuf2 = pMemBuf + MAX_PARAM_BUFFER_SIZE;

	strcpy(pTmpBuf1, pTag);
	strcat(pTmpBuf1, "=");

	// Search target
	if ((ptrStart = strstr_(pOffset, pTmpBuf1)) == NULL)
	{
		DBG_ERR("The target can't be found.\n");
		kfree(pMemBuf);
		return FALSE;
	}

	// We use g_config from insmod module_params as first priority.
	// Be careful for the limitation: Not support blank character.
	if(strcmp(g_config, ""))
	{
                if((ptrEnd = strstr_(ptrStart, " ")) == NULL)
                        ptrEnd = ptrStart + strlen(ptrStart);
	}
	else
	{
		if((ptrEnd = strstr_(ptrStart, "\n")) == NULL)
			ptrEnd = ptrStart + strlen(ptrStart);
	}

	if (ptrEnd < ptrStart)
	{
                DBG_ERR("Point to an outer memory.\n");
		kfree(pMemBuf);
		return FALSE;
	}

	memmove(pTmpBuf2, ptrStart, (ptrEnd - ptrStart));
	pTmpBuf2[ptrEnd - ptrStart] = '\0';
	if((ptrStart = strstr_(pTmpBuf2, "=")) == NULL)
	{
                DBG_ERR("Config file format is wrong.\n");
		kfree(pMemBuf);
		return FALSE;
	}

	ptrValue = ptrStart + 1;
	iLen = strlen(ptrStart);

	memset(pDest, 0, iDestsize);
	strncpy(pDest, ptrValue, ((iLen >= iDestsize) ? iDestsize: iLen));
	kfree(pMemBuf);

	return TRUE;
}

/*!**************************************************************************
* Get the key value of certain target while the target included multiple parameters.
*****************************************************************************/
void motor_get_keyvalue(INT iKey, PCHAR pValue)
{
	INT iIndex;
	PCHAR pKey;

	DBG(emDebugMsg ,"motor_get_keyvalue %d from %s.\n", iKey, pValue);
	for(iIndex = 0, pKey = strtok_(pValue,";"); pKey; pKey = strtok_(NULL,";"), iIndex++)
	{
		if (iKey == iIndex)
		{
			break;
		}
	}
	memmove(pValue, pKey, strlen(pKey)+strlen("\n"));
}

