/*
*******************************************************************************
* $Header: $
*
*  Copyright (c) 2000-2018 VIVOTEK Inc. All rights reserved.
*
*  +-----------------------------------------------------------------+
*  | THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY ONLY BE USED |
*  | AND COPIED IN ACCORDANCE WITH THE TERMS AND CONDITIONS OF SUCH  |
*  | A LICENSE AND WITH THE INCLUSION OF THE THIS COPY RIGHT NOTICE. |
*  | THIS SOFTWARE OR ANY OTHER COPIES OF THIS SOFTWARE MAY NOT BE   |
*  | PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY OTHER PERSON. THE   |
*  | OWNERSHIP AND TITLE OF THIS SOFTWARE IS NOT TRANSFERRED.        |
*  |                                                                 |
*  | THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT   |
*  | ANY PRIOR NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY |
*  | VIVOTEK INC.                                                    |
*  +-----------------------------------------------------------------+
*
* $History: $
*/

/*!
*******************************************************************************
* Copyright 2000-2018 VIVOTEK, Inc. All rights reserved.
*
* \file
* motor_cfg.h
*
* \brief
* Central header file to maintain all include files for motor device driver.
*
*
* \date
* 2018/03/31
*
* \author
* Spark Hsieh <spark.hsieh@vivotek.com>
*
*******************************************************************************
*/

/* ============================================================================= */
#ifndef _MOTOR_CFG_H_
#define _MOTOR_CFG_H_

// Include linux kernel header files
#include <linux/version.h>		// KERNEL_VERSION
#include <linux/kernel.h>		// pr_info
#include <linux/module.h>		// module
#include <linux/init.h>			// modules
#include <linux/moduleparam.h>		// module_param
#include <linux/fs.h>			// chrdev allocation
#include <linux/cdev.h>
#include <linux/device.h>
#include <linux/stddef.h>		// container_of will use offsetof
#include <linux/io.h>			// memset()
#include <linux/ioctl.h>
#include <linux/interrupt.h>
#include <linux/completion.h>
#include <linux/semaphore.h>

#include <linux/string.h>

// Include hrtimer required files
#include <linux/hrtimer.h>
#include <linux/time.h>			// struct timespec
#include <linux/ktime.h>
#include <linux/delay.h>
#include <linux/jiffies.h>

// Include hardware dependent files
//#include <asm/arch/gpio.h>              // set gpio
#include <asm/uaccess.h>                // change kernel/user space
//#include <asm/hardware.h>
#include <asm-generic/types.h>             // machine ID

#include <asm/io.h>
#include <linux/gpio.h>
#include <linux/slab.h>
#include <linux/sched.h>

// Include common header file
#include "motor_dev.h"
#include "motor_linux.h"

/* ============================================================================= */
#define MOTOR_DRIVER_VERSION		"1.0.0.1"
#define MOTOR_VFS_INTERFACE		"vivotekmotor"
#define MOTOR_MAJOR_DEFAULT		183
#define MOTOR_SYSLOG_ID			"[Vivotekmotor]:"
#define MOTOR_LOG_DATE_TIME     "2018/04/11 17:30:00"
/* ============================================================================= */

#define MAX_MOTOR_STEP			500
#define MAX_INI_BUFFER_SIZE		4096
#define MAX_PARAM_BUFFER_SIZE	2048
#define MAX_PARAM_LENGTH		128
#define MAX_NAME_LENGTH			32
#define MAX_DEVICE_NUM			4

#define MOTOR_FORWARD			TRUE
#define MOTOR_BACKWARD			FALSE
#define MOTOR_BLOCK				TRUE
#define MOTOR_NOBLOCK			FALSE

//for simple_strsol
#define DEC 			10
#define HEX 			16

#define LOCKED   		1
#define RELEASED 		0

#define HEATER_ON 		0
#define HEATER_OFF 		1

#define PIB_ENABLE		0
#define PIB_DISABLE		1

typedef struct _TCtrlRegister{		/** Profile for system control register mapping.*/
	unsigned int	uiCtrlChannel;
	unsigned int	uiCtrlPort;
	unsigned int	uiCtrlPortInfo[MAX_DEVICE_NUM];
}TCtrlRegister;


typedef struct _TMotorDev{				/** Motor device structure.*/
	CHAR	szProjectName[MAX_NAME_LENGTH];	/**< The project name.*/
	CHAR	szLensName[MAX_NAME_LENGTH];	/**< The recognition name of this lens.*/
	CHAR	szCtrlIcName[MAX_NAME_LENGTH];	/**< The name of control IC.*/
	CHAR	szDeviceName[MAX_NAME_LENGTH];	/**< The desired device name.*/
	INT		iDeviceId;						/**< The device minor number.*/
	void*	tCtrlRegister;					/**< Indicate the address of system control register.*/
	INT 	iMotorType;
	DWORD	dwDeviceSelect; 				/**< Motor Device Select.*/
	DWORD	dwSpeed;						/**< Indicate the device speed level.*/
	DWORD	dwSpareSteps;					/**< Record the remaining steps.*/
	DWORD	dwCurrentStep;					/**< Record the current step.*/
	DWORD   dwCtrlPhase;        			/**< Motor conctrl type. (0: micoro, 1: 1-2phase(100%), 2: 1-2phase(70%), 3: 2-2phase)*/
	DWORD   dwCtrlLogic;        			/**< Motor conctrl logic. (0: MOTOR_FORWARD=cw ; 1: MOTOR_FORWARD=ccw)*/
	DWORD   dwPatternMove;      			/**< Move pattern steps.*/
	DWORD	dwPISupport; 					/**< Motor PI Support.*/
	DWORD	dwPISwitch; 					/**< Motor PI Switch.*/
	DWORD   dwPICtrlLogic;        			/**< Motor PI conctrl logic. (0: MOTOR_FORWARD=cw ; 1: MOTOR_FORWARD=ccw)*/
	DWORD   dwTotalStep;        			/**< Motor total step*/
	BOOL	bDirection;						/**< Record the device's moving direction.*/
	BOOL	bDirectionPre;					/**< Record the direction info.*/
	BOOL	bFirstStep;						/**< Record the step info.*/
	BOOL	bBlockMode;						/**< Indicate the block behavior.*/
	spinlock_t motorLock;					/**< A lock scheme for data protection.*/
	struct completion completion;			/**< Interrupt waitting of block mode.*/

	/* Dependent Peripheral Control */
	void* 	tHeaterCtrlReg;
	void* 	tPICtrlReg;
	void* 	tPIPowerReg;
}TMotorDev;

typedef enum
{
	eZoom = 0,
	eFocus = 1,
	ePIris = 2,
}EMOTOR_TYPE;

typedef enum
{
	eMicoroStep 	= 0,
	e1_2phase_100 	= 1,
	e1_2phase_70	= 2,
	e2_2phase		= 3,
}EDRIVING_MODE;

// Common API
INT	motor_init_params(TMotorDev *ptDevice);
INT	motor_get_target(PCHAR pTag, PCHAR pDest, INT iDestsize, PCHAR pSrc);
void motor_get_keyvalue(INT iKey, PCHAR pValue);

void pt5111_init_cfg(void);
INT pt5111_start(TMotorDev *ptDevice);

extern void hi_ssp_enable(void);
extern void hi_ssp_disable(void);
extern int hi_ssp_sel_cs(unsigned int uiSpics);
extern void hi_ssp_set_frameform(unsigned char framemode,unsigned char spo,unsigned char sph,unsigned char datawidth);
extern void hi_ssp_set_serialclock(unsigned char scr,unsigned char cpsdvsr);
extern void hi_ssp_alt_mode_set(int enable);
extern void hi_ssp_set_spi4_reverse(void);
extern void spi_enable(void);
extern void spi_disable(void);
extern void spi_write_a16byte(unsigned short spi_dat, unsigned int spi_cs);
#endif

