/*
*******************************************************************************
* $Header: $
*
*  Copyright (c) 2000-2018 VIVOTEK Inc. All rights reserved.
*
*  +-----------------------------------------------------------------+
*  | THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY ONLY BE USED |
*  | AND COPIED IN ACCORDANCE WITH THE TERMS AND CONDITIONS OF SUCH  |
*  | A LICENSE AND WITH THE INCLUSION OF THE THIS COPY RIGHT NOTICE. |
*  | THIS SOFTWARE OR ANY OTHER COPIES OF THIS SOFTWARE MAY NOT BE   |
*  | PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY OTHER PERSON. THE   |
*  | OWNERSHIP AND TITLE OF THIS SOFTWARE IS NOT TRANSFERRED.        |
*  |                                                                 |
*  | THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT   |
*  | ANY PRIOR NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY |
*  | VIVOTEK INC.                                                    |
*  +-----------------------------------------------------------------+
*
* $History: $
*/

/*!
*******************************************************************************
* Copyright 2000-2018 VIVOTEK, Inc. All rights reserved.
*
* \file
* motor_dev.h
*
* \brief
* Header file of motor control driver which user-api needs to include.
*
* \date
* 2018/03/31
*
* \author
* Spark Hsieh <spark.hsieh@vivotek.com>
*
*******************************************************************************
*/

/* ============================================================================= */
#ifndef _MOTOR_DEV_H_
#define _MOTOR_DEV_H_

#include "typedef.h"

/** Debug Level*/
typedef enum {
	emDebugOff	= 0,
	emDebugMsg	= 1,
	emDebugInfo	= 2,
} EMotorDebug;

/** Motor Speed Level, uint: pps */
#define MOTOR_OPERATING_BUF_TIME		15 / 10 //multiplied 1.5
#define MOTOR_PRE_POST_EXCITATION		20000	//8.192ms * 2
#define MOTOR_MIN_SPEED					100
#define MOTOR_START_SPEED				MOTOR_MIN_SPEED
#define MOTOR_LOWER_SPEED				400
#define MOTOR_NORMAL_SPEED				600
#define MOTOR_DEFAULT_SPEED				1500	//for MOTOR_DEFAULT_SPEED spec
#define MOTOR_HIGHER_SPEED				600
#define MOTOR_SUPER_SPEED				800
#define MOTOR_MAX_SPEED					2000	//for MOTOR_DEFAULT_SPEED spec

/** user 'M' as magic number M=0x4D */
#define MOTOR_MAGIC		'M'

/** Set the motor forward direction */
#define MOTOR_SET_FORWARD			_IO (MOTOR_MAGIC, 0)
/** Set the motor backward direction */
#define MOTOR_SET_BACKWARD			_IO (MOTOR_MAGIC, 1)
/** Set the speed value of motor */
#define MOTOR_SET_SPEED				_IOW (MOTOR_MAGIC, 2, DWORD)
/** Set the walk steps value of motor (non-blocking mode)*/
#define MOTOR_WALK_STEPS			_IOW (MOTOR_MAGIC, 3, DWORD)
/** Clear the current walk steps value of motor */
#define MOTOR_WALK_STEPS_CLR		_IO (MOTOR_MAGIC, 4)
/** Turn off motors for thermal issue */
#define MOTOR_TURN_OFF				_IO (MOTOR_MAGIC, 5)
/** Set the walk steps value of motor (blocking mode)*/
#define MOTOR_WALK_STEPS_BLOCK		_IOW (MOTOR_MAGIC, 6, DWORD)
/** Get the remaining steps of motor */
#define MOTOR_REMAIN_STEPS			_IO (MOTOR_MAGIC, 7)
/** Search PI position of motor */
#define MOTOR_SEARCH_PI				_IOW (MOTOR_MAGIC, 8, DWORD)
/** Get PI status(high / low) */
#define MOTOR_GET_PI_STATUS			_IO (MOTOR_MAGIC, 9)
/** Set SPI cs sel */
#define MOTOR_CHECK_PI_WALK_STEPS	_IOW (MOTOR_MAGIC, 10, DWORD)
/** Set the debug message level of driver status */
#define MOTOR_SET_MSG_LEVEL			_IOW (MOTOR_MAGIC, 11, DWORD)
/** List the current system log of driver status */
#define MOTOR_SYS_LOG_LIST			_IOW (MOTOR_MAGIC, 12, DWORD)
/** Set SPI cs sel */
#define MOTOR_DEVICE_SELECT			_IOW (MOTOR_MAGIC, 13, DWORD)
/** Set PI power on */
#define MOTOR_SET_PI_POWER_ON		_IO (MOTOR_MAGIC, 14)
/** Set PI power off */
#define MOTOR_SET_PI_POWER_OFF		_IO (MOTOR_MAGIC, 15)
/** Stop Search PI position of motor */
#define MOTOR_STOP_SEARCH_PI		_IO (MOTOR_MAGIC, 16)

/** The max number of ioctl's */
#define MOTOR_MAXNR 16

#endif
