/*
*******************************************************************************
* $Header: $
*
*  Copyright (c) 2000-2018 VIVOTEK Inc. All rights reserved.
*
*  +-----------------------------------------------------------------+
*  | THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY ONLY BE USED |
*  | AND COPIED IN ACCORDANCE WITH THE TERMS AND CONDITIONS OF SUCH  |
*  | A LICENSE AND WITH THE INCLUSION OF THE THIS COPY RIGHT NOTICE. |
*  | THIS SOFTWARE OR ANY OTHER COPIES OF THIS SOFTWARE MAY NOT BE   |
*  | PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY OTHER PERSON. THE   |
*  | OWNERSHIP AND TITLE OF THIS SOFTWARE IS NOT TRANSFERRED.        |
*  |                                                                 |
*  | THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT   |
*  | ANY PRIOR NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY |
*  | VIVOTEK INC.                                                    |
*  +-----------------------------------------------------------------+
*
* $History: $
*/

/*!
*******************************************************************************
* Copyright 2000-2018 VIVOTEK, Inc. All rights reserved.
*
* \file
* motor_linux.c
*
* \brief
* The implementation of linux version dependence kernel api.
*
* \date
* 2018/03/31
*
* \author
* Spark Hsieh <spark.hsieh@vivotek.com>
*
*******************************************************************************
*/

#include "motor_cfg.h"

/******************************************************************************
* File open/close/read related functions.	
*****************************************************************************/
MOTOR_OS_FD motor_osfs_open(PCHAR pPath,  INT flag, INT mode)
{
	struct file *filePtr;
		
	filePtr = filp_open(pPath, flag, 0);
	if (IS_ERR(filePtr))
	{
		DBG_ERR("%s(): Error %ld opening %s\n", __FUNCTION__, -PTR_ERR(filePtr), pPath);
	}

	return (MOTOR_OS_FD)filePtr;
}

INT motor_osfs_close(MOTOR_OS_FD osfd)
{
	filp_close(osfd, NULL);
	return 0;
}

INT motor_osfs_read(MOTOR_OS_FD osfd, char *pDataPtr, int readLen)
{
	if (osfd->f_op && osfd->f_op->read)
	{
		return osfd->f_op->read(osfd,  pDataPtr, readLen, &osfd->f_pos);
	}
	else
	{
		DBG_ERR("Loss file read method\n");
		return -1;
	}
}

/****************************************************************************
 * Split a string by tokens
 * @s: The input string
 * @ct: The token characters
 * WARNING: strtok is deprecated, use strsep instead, but strsep is not compatible with old architecture.
*****************************************************************************/
PCHAR __strtok1;
PCHAR strtok_(PCHAR s,const PCHAR ct)
{
	PCHAR sbegin, send;

	sbegin  = s ? s : __strtok1;
	if (!sbegin)
	{
		return NULL;
	}

	sbegin += strspn(sbegin,ct);
	if (*sbegin == '\0')
	{
		__strtok1 = NULL;
		return( NULL );
	}

	send = strpbrk( sbegin, ct);
	if (send && *send != '\0')
		*send++ = '\0';

	__strtok1 = send;

	return (sbegin);
}

PCHAR __strtok2;
PCHAR strtok__(PCHAR s,const PCHAR ct)
{
        PCHAR sbegin, send;

        sbegin  = s ? s : __strtok2;
        if (!sbegin)
        {
                return NULL;
        }

        sbegin += strspn(sbegin,ct);
        if (*sbegin == '\0')
        {
                __strtok2 = NULL;
                return( NULL );
        }

        send = strpbrk( sbegin, ct);
        if (send && *send != '\0')
                *send++ = '\0';

        __strtok2 = send;

        return (sbegin);
}

/****************************************************************************
 * Find the first substring in an input string
 * @s1: The string to be searched
 * @s2: The string to search for
*****************************************************************************/
PCHAR strstr_(PCHAR s1,const PCHAR s2)
{
	INT l1, l2;

	l2 = strlen(s2);
	if (!l2) return s1;
	
	l1 = strlen(s1);
	while (l1 >= l2)
	{
		l1--;
		if (!memcmp(s1,s2,l2)) return s1;
		s1++;
	}
	
	return NULL;
}

/******************************************************************************
* Linux Version Dependency, only for  kernel version 2.6.18
*****************************************************************************/
#if (LINUX_VERSION_CODE == KERNEL_VERSION(2,6,18))	
ktime_t Linux_ktime_get(void)
{
        struct timespec now;

        ktime_get_ts(&now);

        return timespec_to_ktime(now);
}

unsigned long Linux_ktime_divns(const ktime_t kt, s64 div)
{
        u64 dclc, inc, dns;
        int sft = 0;

        dclc = dns = ktime_to_ns(kt);
        inc = div;
        /* Make sure the divisor is less than 2^32: */
        while (div >> 32) {
                sft++;
                div >>= 1;
        }
        dclc >>= sft;
        do_div(dclc, (unsigned long) div);

        return (unsigned long) dclc;
}

ktime_t Linux_ktime_add_ns(const ktime_t kt, u64 nsec)
{
        ktime_t tmp;

        if (likely(nsec < NSEC_PER_SEC)) {
                tmp.tv64 = nsec;
        } else {
                unsigned long rem = do_div(nsec, NSEC_PER_SEC);

                tmp = ktime_set((long)nsec, rem);
        }

        return ktime_add(kt, tmp);
}

unsigned long Linux_hrtimer_forward(struct hrtimer *timer, ktime_t now, ktime_t interval)
{
        unsigned long orun = 1;
        ktime_t delta;

        delta = ktime_sub(now, timer->expires);

        if (delta.tv64 < 0)
                return 0;

        if (interval.tv64 < timer->base->resolution.tv64)
                interval.tv64 = timer->base->resolution.tv64;

        if (unlikely(delta.tv64 >= interval.tv64)) {
                s64 incr = ktime_to_ns(interval);

                orun = Linux_ktime_divns(delta, incr);
                timer->expires = Linux_ktime_add_ns(timer->expires, incr * orun);
                if (timer->expires.tv64 > now.tv64)
                        return orun;
                /*
                 * This (and the ktime_add() below) is the
                 * correction for exact:
                 */
                orun++;
        }
        timer->expires = ktime_add(timer->expires, interval);
        /*
         * Make sure, that the result did not wrap with a very large
         * interval.
         */
        if (timer->expires.tv64 < 0)
                timer->expires = ktime_set(KTIME_SEC_MAX, 0);

        return orun;
}
#endif //#if (LINUX_VERSION_CODE == KERNEL_VERSION(2,6,18))	
