/*
*******************************************************************************
* $Header: $
*
*  Copyright (c) 2000-2018 VIVOTEK Inc. All rights reserved.
*
*  +-----------------------------------------------------------------+
*  | THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY ONLY BE USED |
*  | AND COPIED IN ACCORDANCE WITH THE TERMS AND CONDITIONS OF SUCH  |
*  | A LICENSE AND WITH THE INCLUSION OF THE THIS COPY RIGHT NOTICE. |
*  | THIS SOFTWARE OR ANY OTHER COPIES OF THIS SOFTWARE MAY NOT BE   |
*  | PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY OTHER PERSON. THE   |
*  | OWNERSHIP AND TITLE OF THIS SOFTWARE IS NOT TRANSFERRED.        |
*  |                                                                 |
*  | THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT   |
*  | ANY PRIOR NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY |
*  | VIVOTEK INC.                                                    |
*  +-----------------------------------------------------------------+
*
* $History: $
*/

/*!
*******************************************************************************
* Copyright 2000-2018 VIVOTEK, Inc. All rights reserved.
*
* \file
* motor_linux.h
*
* \brief
* The header file contains linux version dependence kernel api.
*
*
* \date
* 2018/03/31
*
* \author
* Spark Hsieh <spark.hsieh@vivotek.com>
*
*******************************************************************************
*/

/* ============================================================================= */
#ifndef _MOTOR_LINUX_H_
#define _MOTOR_LINUX_H_

extern BYTE g_Debug;
#define DBG_ERR(Fmt, ...) pr_err(Fmt, ##__VA_ARGS__)
#define DBG(Level, Fmt, ...)						\
do{									\
	if (Level <= g_Debug)						\
	{								\
			printk("MotorDriver: "Fmt, ##__VA_ARGS__);	\
	}								\
}while(0)
/* /proc/sys/kernel/printk_ratelimit: the trylater time, default 5 sec.			*/
/* /proc/sys/kernel/printk_ratelimit_burst: the bound of queue numbers, default 10.	*/

/* Lock set */
#define MOTOR_LOCK_INIT(_pLock)			spin_lock_init(_pLock)
#define MOTOR_LOCK_GET(_pLock, _flags)		spin_lock_irqsave(_pLock, _flags)
#define MOTOR_LOCK_RELEASE(_pLock, _flags)	spin_unlock_irqrestore(_pLock, _flags)

/* Kernel wait*/
#define MOTOR_INIT_INTERRUPT(_pComp)		init_completion(_pComp)
#define MOTOR_CLR_INTERRUPT(_pComp)		init_completion(_pComp)
#define MOTOR_WAIT_INTERRUPT(_pComp)		wait_for_completion_interruptible(_pComp) // Waiting for steps to complete
#define MOTOR_NOTIFY_INTERRUPT(_pComp)		complete(_pComp) // Notify first process who is wait for completion

typedef struct file* MOTOR_OS_FD;

typedef struct _MOTOR_OS_FS_INFO_{
	int		fsuid;
	int		fsgid;
	mm_segment_t	fs;
}MOTOR_OS_FS_INFO;

MOTOR_OS_FD	motor_osfs_open(PCHAR pPath,  INT flag, INT mode);
INT		motor_osfs_close(MOTOR_OS_FD osfd);
INT		motor_osfs_read(MOTOR_OS_FD osfd, PCHAR pDataPtr, INT readLen);
PCHAR		strtok_(PCHAR s,const PCHAR ct);
PCHAR		strtok__(PCHAR s,const PCHAR ct);
PCHAR		strstr_(PCHAR s1,const PCHAR s2);
ktime_t 	Linux_ktime_get(void);
unsigned long	Linux_ktime_divns(const ktime_t kt, s64 div);
ktime_t		Linux_ktime_add_ns(const ktime_t kt, u64 nsec);
unsigned long	Linux_hrtimer_forward(struct hrtimer *timer, ktime_t now, ktime_t interval);

#endif
