/*
 *******************************************************************************
 * $Header: $
 *
 *  Copyright (c) 2000-2018 VIVOTEK Inc. All rights reserved.
 *
 *  +-----------------------------------------------------------------+
 *  | THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY ONLY BE USED |
 *  w
 *  | AND COPIED IN ACCORDANCE WITH THE TERMS AND CONDITIONS OF SUCH  |
 *  | A LICENSE AND WITH THE INCLUSION OF THE THIS COPY RIGHT NOTICE. |
 *  | THIS SOFTWARE OR ANY OTHER COPIES OF THIS SOFTWARE MAY NOT BE   |
 *  | PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY OTHER PERSON. THE   |
 *  | OWNERSHIP AND TITLE OF THIS SOFTWARE IS NOT TRANSFERRED.        |
 *  |                                                                 |
 *  | THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT   |
 *  | ANY PRIOR NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY |
 *  | VIVOTEK INC.                                                    |
 *  +-----------------------------------------------------------------+
 *
 * $History: $
 */

/*!
 *******************************************************************************
 * Copyright 2000-2018 VIVOTEK, Inc. All rights reserved.
 *
 * \file
 * motor_main.c
 *
 * \brief
 * Source file of motor control driver.
 *
 * \date
 * 2018/03/31
 *
 * \author
 * Spark Hsieh <spark.hsieh@vivotek.com>
 *
 *******************************************************************************
 */

#include "motor_cfg.h"

BYTE g_Debug = emDebugInfo; // debug message level;

volatile char rcsid[] = "$Id: " MOTOR_DRIVER_VERSION " zoom-motor-driver " MOTOR_LOG_DATE_TIME " $";

/* The essential info while init/exit module */
static BOOL bSearchPIGoing = FALSE;
static struct class *motorclass = NULL;
static INT allocatedTag = 0x00;
static INT isopenedTag = 0x00;

/* Get arg parameters by insmod */
static PCHAR g_devname = MOTOR_VFS_INTERFACE;
module_param(g_devname, charp, S_IRUGO);	// Get the dev name from argument
static UINT g_devmajor = MOTOR_MAJOR_DEFAULT;
module_param(g_devmajor, uint, S_IRUGO);	// Get the dev major number from argument
static PCHAR g_devminor = "0";
module_param(g_devminor, charp, S_IRUGO);	// Get the dev minnor number from argument
PCHAR g_config = "";
module_param(g_config, charp, S_IRUGO);		// Get the config content included all parameters

/* Declare local functions */
static INT motor_open (struct inode *inode, struct file *filp);
static INT motor_release (struct inode *inode, struct file *filp);
// KERNEL_VERSION > (2,6,36)
static long motor_ioctl (struct file *filp, unsigned int cmd, unsigned long  dwArg);
static INT motor_init_module(void);
static void motor_cleanup_module(void);

TMotorDev* motor_new_device(void);

static UINT motor_trans_speed(DWORD dwspeed);

/*!**************************************************************************
 * Set motor direction to forward.
 *****************************************************************************/
INT motor_set_forward(TMotorDev *ptDevice)
{
	ULONG ulFlags;

	MOTOR_LOCK_GET(&ptDevice->motorLock, ulFlags);
	ptDevice->bDirection = MOTOR_FORWARD;
	MOTOR_LOCK_RELEASE(&ptDevice->motorLock, ulFlags);

	return S_OK;
}

/*!***************************************************************************
 * Set motor direction to backward.
 *****************************************************************************/
INT motor_set_backward(TMotorDev *ptDevice)
{
	ULONG ulFlags;

	MOTOR_LOCK_GET(&ptDevice->motorLock, ulFlags);
	ptDevice->bDirection = MOTOR_BACKWARD;
	MOTOR_LOCK_RELEASE(&ptDevice->motorLock, ulFlags);

	return S_OK;
}

/*!**************************************************************************
 * Set motor speed level.
 *****************************************************************************/
INT motor_set_speed(TMotorDev *ptDevice, unsigned long ulSpeed)
{
	ULONG ulFlags;

	MOTOR_LOCK_GET(&ptDevice->motorLock, ulFlags);

	if (ulSpeed > MOTOR_MAX_SPEED)
	{
		ulSpeed = MOTOR_MAX_SPEED;
	}
	else if (ulSpeed < MOTOR_MIN_SPEED)
	{
		ulSpeed = MOTOR_MIN_SPEED;
	}

	ptDevice->dwSpeed = ulSpeed;
	DBG(emDebugMsg, "Set motor_set_speed :%d pps. \n", ptDevice->dwSpeed);

	MOTOR_LOCK_RELEASE(&ptDevice->motorLock, ulFlags);

	return S_OK;
}

/*!**************************************************************************
 * Stop motor stepping and return the remaining steps.
 *****************************************************************************/
INT motor_set_clean(TMotorDev *ptDevice)
{
	ULONG ulFlags;
	INT iSteps = 0;

	MOTOR_LOCK_GET(&ptDevice->motorLock, ulFlags);

	if (ptDevice->dwSpareSteps > 0)
	{
		iSteps = ptDevice->dwSpareSteps - 1;
		ptDevice->dwSpareSteps = 1; // Clear the current walk steps. Forced to set the remaining to 1 step.
	}

	DBG(emDebugMsg, "Set motor_set_clean : suspend %d steps.\n", iSteps);

	MOTOR_LOCK_RELEASE(&ptDevice->motorLock, ulFlags);

	return iSteps;
}

/*!**************************************************************************
 * Set motor to stepping mode and decide the block method.  (bBlock - 0: no-block, 1: block)
 *****************************************************************************/
INT motor_walk_steps(TMotorDev *ptDevice, DWORD steps, BOOL bBlock)
{
	ULONG ulFlags;
	unsigned int uisleep_time = 0;
	BOOL bLock = RELEASED;

	if (steps <= 0)
	{
		return steps;
	}

	DBG(emDebugMsg, "Trigger motor %s to walk %d steps.\n", ptDevice->szDeviceName, steps);

	MOTOR_LOCK_GET(&ptDevice->motorLock, ulFlags);
    bLock = LOCKED;

	// Update setting
	ptDevice->bFirstStep = TRUE;
	ptDevice->dwSpareSteps = steps;
	ptDevice->bBlockMode = bBlock;

	//PPS * Steps + pre excitation + post excitation
	uisleep_time = (motor_trans_speed(ptDevice->dwSpeed) * ptDevice->dwSpareSteps)+ MOTOR_PRE_POST_EXCITATION;
	uisleep_time = uisleep_time * MOTOR_OPERATING_BUF_TIME;
	uisleep_time = (uisleep_time /1000) + 1;
	pt5111_start(ptDevice);

	msleep(uisleep_time);

	if(bLock == LOCKED)
	{
		MOTOR_LOCK_RELEASE(&ptDevice->motorLock, ulFlags);
	}


	return ptDevice->dwSpareSteps;
}

/*!**************************************************************************
* Init PIB pin.
*****************************************************************************/
INT pib_register_init(unsigned int uiCtrlPort)
{
	if (gpio_direction_input(uiCtrlPort) != 0)
	{
		return S_FAIL;
	}
	
	return S_OK;
}

/*!**************************************************************************
* Free PIB pin.
*****************************************************************************/
void pib_register_free(unsigned int uiCtrlPort)
{
	gpio_free(uiCtrlPort);
}

/*!**************************************************************************
* Check PIB pin status
*****************************************************************************/
INT pib_monitor(TMotorDev *ptDevice)
{
	TCtrlRegister*	tPIRegister = (TCtrlRegister*)(ptDevice->tPICtrlReg);
	int iValue;

	if (!ptDevice->dwPISupport)
	{
		DBG(emDebugInfo, "not support PI monitor\n");
		return S_FAIL;
	}
	
	if (pib_register_init(tPIRegister->uiCtrlPort) != S_OK)
	{
		DBG(emDebugInfo, "pib_register_init fail\n");
		return S_FAIL;
	}

	iValue = gpio_get_value(tPIRegister->uiCtrlPort);
	//printk("PI port %u Value: %d\n", tPIRegister->uiCtrlPort, iValue);
	pib_register_free(tPIRegister->uiCtrlPort);

	return iValue;
}

/*!**************************************************************************
* Search PI position of motor
*****************************************************************************/
INT Get_PI_status(TMotorDev *ptDevice)
{
	unsigned int uiData = 0;
	ULONG ulFlags;

	MOTOR_LOCK_GET(&ptDevice->motorLock, ulFlags);
	uiData = pib_monitor(ptDevice);
	MOTOR_LOCK_RELEASE(&ptDevice->motorLock, ulFlags);

	return uiData;
}

/*!**************************************************************************
* Init PI Power pin.
*****************************************************************************/
INT pi_power_register_init(unsigned int uiCtrlPort)
{
	if (gpio_direction_output(uiCtrlPort, 0) != 0)
	{
		return S_FAIL;
	}

	return S_OK;
}

/*!**************************************************************************
* Free PI Power pin.
*****************************************************************************/
void pi_power_register_free(unsigned int uiCtrlPort)
{
	gpio_free(uiCtrlPort);
}

/*!**************************************************************************
* Init PI switch
*****************************************************************************/
INT Init_PI_Switch(TMotorDev *ptDevice)
{
	TCtrlRegister*	tPIPowerRegister = (TCtrlRegister*)(ptDevice->tPIPowerReg);

	if (pi_power_register_init(tPIPowerRegister->uiCtrlPort) != S_OK)
	{
		DBG(emDebugInfo, "pi_power_register_init fail\n");
		return S_FAIL;
	}

	return S_OK;
}

/*!**************************************************************************
* Set PI switch
*****************************************************************************/
INT Set_PI_Switch(TMotorDev *ptDevice, BOOL bPowerOn)
{
	TCtrlRegister*	tPIPowerRegister = (TCtrlRegister*)(ptDevice->tPIPowerReg);
	//DBG(emDebugInfo, "Set_PI_Switch port:%d, Power:%d\n", tPIPowerRegister->uiCtrlPort, bPowerOn);
	gpio_set_value(tPIPowerRegister->uiCtrlPort, bPowerOn);

	return S_OK;
}

/*!**************************************************************************
* Release PI switch
*****************************************************************************/
INT Release_PI_Switch(TMotorDev *ptDevice)
{
	TCtrlRegister*	tPIPowerRegister = (TCtrlRegister*)(ptDevice->tPIPowerReg);

	pib_register_free(tPIPowerRegister->uiCtrlPort);

	return S_OK;
}

/*!**************************************************************************
* Search PI position of motor
*****************************************************************************/
INT Search_PI_position(TMotorDev *ptDevice, DWORD steps, BOOL bBlock)
{
	BYTE		 tmp_Debug;
	unsigned int uiOriData = 0;
	unsigned int uiNewData = 0;
	unsigned int uisleep_time = 1000;
	int iCount = 0;

	if (!ptDevice->dwPISwitch)
	{
		DBG(emDebugInfo, "Not support PI\n");
		return S_FAIL;
	}

	if (steps <= 0)
	{
		steps = 1;
	}

	msleep(uisleep_time);

	uiOriData = pib_monitor(ptDevice);

	if (uiOriData == 1)
	{
		motor_set_forward(ptDevice);
	}
	else
	{
		motor_set_backward(ptDevice);
	}

	tmp_Debug = g_Debug;
	g_Debug = emDebugOff;
	bSearchPIGoing = TRUE;

	while (bSearchPIGoing)
	{
		iCount++;

		motor_walk_steps(ptDevice, steps, bBlock);

		//Set_PI_Switch(ptDevice, TRUE);
		uiNewData = pib_monitor(ptDevice);

		//printk("uiOriData = %u, uiNewData = %u\n", uiOriData, uiNewData);
		if (uiNewData != uiOriData)
		{
			printk("Search PI Done iCount = %d, uiNewData = %d, uiOriData = %d\n", iCount, uiNewData, uiOriData);

			g_Debug = tmp_Debug;
			return S_OK;
		}

		if (iCount >= ptDevice->dwTotalStep)
		{	
			printk("Search PI fail over iCount = %d\n", iCount);
			break;
		}
	}

	g_Debug = tmp_Debug;
	bSearchPIGoing = FALSE;

	return S_OK;
}

/*!**************************************************************************
* Stop Search PI position
*****************************************************************************/
INT Stop_Search_PI(TMotorDev *ptDevice)
{
	bSearchPIGoing = FALSE;
	return S_OK;
}


/*!**************************************************************************
* Select SPI CS pin
*****************************************************************************/
int Set_SPI_CS_PIN(TMotorDev *ptDevice ,unsigned long ulSpics)
{
	TCtrlRegister*	tPIRegister = (TCtrlRegister*)(ptDevice->tPICtrlReg);
	SCODE scResult;
	ULONG ulFlags;

	MOTOR_LOCK_GET(&ptDevice->motorLock, ulFlags);

	tPIRegister->uiCtrlPort = tPIRegister->uiCtrlPortInfo[ulSpics];
	ptDevice->dwDeviceSelect = ulSpics;

	//DBG(emDebugMsg, "Select dev %lu PI CtrlPort %u. \n", ulSpics, tPIRegister->uiCtrlPort);
	scResult = hi_ssp_sel_cs(ulSpics);

	MOTOR_LOCK_RELEASE(&ptDevice->motorLock, ulFlags);

	return scResult;
}

/*!**************************************************************************
* Set PI power on
*****************************************************************************/
int Set_PI_Power_ON(TMotorDev *ptDevice)
{
	SCODE scResult;

	scResult = Init_PI_Switch(ptDevice);
	if (Init_PI_Switch(ptDevice) != S_OK)
	{
		return S_FAIL;
	}

	if (Set_PI_Switch(ptDevice, TRUE) != S_OK)
	{
		return S_FAIL;
	}

	return S_OK;
}

/*!**************************************************************************
* Set PI power off
*****************************************************************************/
int Set_PI_Power_OFF(TMotorDev *ptDevice)
{
	SCODE scResult;

	scResult = Init_PI_Switch(ptDevice);
	if (Init_PI_Switch(ptDevice) != S_OK)
	{
		return S_FAIL;
	}

	if (Set_PI_Switch(ptDevice, FALSE) != S_OK)
	{
		return S_FAIL;
	}

	if (Release_PI_Switch(ptDevice) != S_OK)
	{
		return S_FAIL;
	}

	return S_OK;
}

/*!**************************************************************************
 * Print the motor system log.
 *****************************************************************************/
INT motor_system_log(TMotorDev *ptDevice, DWORD dwLogLevel)
{
	printk("MotorDriver Info (%s) -- \n \
			ProjectName: %s, LensName: %s, CtrlIcName: %s, \n \
			Current Direction: %s, \n \
			Current Speed: %d (pps).\n ", ptDevice->szDeviceName
			, ptDevice->szProjectName, ptDevice->szLensName, ptDevice->szCtrlIcName
			,(ptDevice->bDirection) ? "Forward": "Backward"
			, ptDevice->dwSpeed);
	return S_OK;
}



/*!**************************************************************************
 * New the device structure while open device.
 *****************************************************************************/
TMotorDev* motor_new_device()
{
	TMotorDev* ptDevice = NULL;

	ptDevice = kmalloc(sizeof(TMotorDev), GFP_ATOMIC); // kmalloc for motor device structure used.
	if (!ptDevice)
	{
		DBG_ERR("%s New device failed - No memory !!\n", MOTOR_SYSLOG_ID);
		return ptDevice;
	}

	memset(ptDevice, 0x00, sizeof(TMotorDev));

	return ptDevice;
}

/*!**************************************************************************
 * Transform the speed unit from pps to delay time(us).
 *****************************************************************************/
static UINT motor_trans_speed(DWORD dwSpeed)
{
	// +10% buffer
	return ((1000000 / dwSpeed) + (100000 / dwSpeed));	// Transform pps to delay time (us).
}

/*!**************************************************************************
 * Initialize desired device by minor number while open device.
 *****************************************************************************/
static int motor_open (struct inode *inode, struct file *filp)
{
	TMotorDev *ptMotorDev = NULL;

/*    DBG(emDebugMsg, "%s: major %d minor %d (pid %d)\n", __func__, imajor(inode), iminor(inode), current->pid);*/

	if((isopenedTag >> iminor(inode)) & 0x01)
	{
		DBG_ERR("%s The device was occupied.\n", MOTOR_SYSLOG_ID);
		return S_FAIL;
	}
	else
	{
		//New a desired device
		ptMotorDev = motor_new_device();
		if (!ptMotorDev)
		{
			DBG_ERR("%s Open device failed.\n", MOTOR_SYSLOG_ID);
			return S_FAIL;
		}
		else
		{
			isopenedTag |= (0x01 << iminor(inode));
			ptMotorDev->iDeviceId = iminor(inode);
			DBG(emDebugInfo, "New device success (%d, %d).\n", imajor(inode), ptMotorDev->iDeviceId);
		}
	}

	// initial the parameters from config file
	if (motor_init_params(ptMotorDev) != S_OK)
	{
		DBG_ERR("%s Open config file failed.\n", MOTOR_SYSLOG_ID);
		kfree(ptMotorDev);
		return S_FAIL;
	}
	ptMotorDev->bDirection = MOTOR_BACKWARD;
	ptMotorDev->bDirectionPre = MOTOR_FORWARD;
	ptMotorDev->bFirstStep = TRUE;
	ptMotorDev->dwCurrentStep = 0;
	ptMotorDev->dwPatternMove = ptMotorDev->dwCtrlPhase ? 1 : 2;


	// initial a semaphore
	MOTOR_LOCK_INIT(&ptMotorDev->motorLock);

	//initial interrupt
	MOTOR_INIT_INTERRUPT(&ptMotorDev->completion);

	filp->private_data = ptMotorDev;

	spi_write_a16byte(0xCFC0, 0);//1100 1111 1100 0000   ps
	spi_write_a16byte(0xCFC0, 1);//1100 1111 1100 0000   ps
	spi_write_a16byte(0xCFC0, 2);//1100 1111 1100 0000   ps
	spi_write_a16byte(0xCFC0, 3);//1100 1111 1100 0000   ps

	DBG(emDebugMsg, "%s Driver Open.\n", MOTOR_SYSLOG_ID);
	return S_OK;

}

/******************************************************************************
 * motor_ioctl - function to implement the GPIO ioctl's
 *****************************************************************************/
// KERNEL_VERSION > (2,6,36)
static long motor_ioctl (struct file *filp, unsigned int cmd, unsigned long dwArg)
{
	TMotorDev *ptDevice = (TMotorDev *)filp->private_data;
	SCODE scResult = 0;

	if ((_IOC_TYPE( cmd ) != MOTOR_MAGIC ) || (_IOC_NR( cmd ) > MOTOR_MAXNR ))
	{
		return -ENOTTY;
	}

	switch (cmd)
	{
		case MOTOR_SET_FORWARD:
			scResult = motor_set_forward(ptDevice);
			break;
		case MOTOR_SET_BACKWARD:
			scResult = motor_set_backward(ptDevice);
			break;
		case MOTOR_SET_SPEED:
			scResult = motor_set_speed(ptDevice, dwArg);
			break;
		case MOTOR_WALK_STEPS:
			//scResult = motor_walk_steps(ptDevice, dwArg, MOTOR_NOBLOCK);
			//!!!abf motor control only support block mode because of "DAC control" and "PIB switch"!!!
			scResult = motor_walk_steps(ptDevice, dwArg, MOTOR_BLOCK);
			break;
		case MOTOR_WALK_STEPS_BLOCK:
			scResult = motor_walk_steps(ptDevice, dwArg, MOTOR_BLOCK);
			break;
		case MOTOR_REMAIN_STEPS:
			scResult = ptDevice->dwSpareSteps;
			break;
		case MOTOR_SEARCH_PI:
			scResult = Search_PI_position(ptDevice, dwArg, MOTOR_BLOCK);
			break;
		case MOTOR_STOP_SEARCH_PI:
			scResult = Stop_Search_PI(ptDevice);
			break;
		case MOTOR_GET_PI_STATUS:
			scResult = Get_PI_status(ptDevice);
			DBG(emDebugMsg, "Get_PI_status = %d\n", scResult);
			break;
		case MOTOR_DEVICE_SELECT:
			scResult = Set_SPI_CS_PIN(ptDevice, dwArg);
			break;
		case MOTOR_SET_PI_POWER_ON:
			scResult = Set_PI_Power_ON(ptDevice);
			break;
		case MOTOR_SET_PI_POWER_OFF:
			scResult = Set_PI_Power_OFF(ptDevice);
			break;
		case MOTOR_WALK_STEPS_CLR:
			scResult = motor_set_clean(ptDevice);
			break;
		case MOTOR_SET_MSG_LEVEL:
			g_Debug = dwArg;
			break;
		case MOTOR_SYS_LOG_LIST:
			scResult = motor_system_log(ptDevice, dwArg);
			break;
		case MOTOR_TURN_OFF:
			//motor_register_clean(ptDevice);
			break;
		default:
			return -ENOTTY;
			break;
	}
	return (scResult);
}

/*!**************************************************************************
 * Release the resource of motor device while close device.
 *****************************************************************************/
static int motor_release(struct inode *inode, struct file *filp)
{
	TMotorDev *ptDevice = (TMotorDev *)filp->private_data;

	if (ptDevice)
	{
		isopenedTag &= (~(0x01 << ptDevice->iDeviceId));

		kfree(ptDevice);
	}

	DBG(emDebugMsg, "%s Motor driver release. \n", MOTOR_SYSLOG_ID);

	return S_OK;
}

/* declare file operation structure */
static struct file_operations motor_fops = {
owner:	THIS_MODULE,
		open:	motor_open,
// KERNEL_VERSION > (2,6,36)
		unlocked_ioctl: motor_ioctl,
		release: motor_release,
};

/*!*************************************************************************
 * Initialize motor module while insmod.
 ****************************************************************************/
int __init motor_init_module ()
{
	struct class_device *device = NULL;

	// Register a char device to kernel
	if (register_chrdev(g_devmajor, MOTOR_VFS_INTERFACE, &motor_fops) < 0)
	{
		DBG_ERR("%s Couln't register the device.", MOTOR_SYSLOG_ID);
		return -EFAULT;
	}

	// Create device class in sysfs
	if (!motorclass)
	{
		motorclass = class_create(THIS_MODULE, MOTOR_VFS_INTERFACE);
		if (IS_ERR(motorclass))
		{
			unregister_chrdev(g_devmajor, MOTOR_VFS_INTERFACE);
			DBG_ERR("%s Class creation failure.\n", MOTOR_SYSLOG_ID);
			return PTR_ERR(motorclass);
		}
	}

	// Create multiple device by g_devname & g_devminor
	do
	{
		PCHAR	pDevname, pDevminor;
		UINT	uiDevminor = 0;

		for (pDevname = strtok_(g_devname, ";"), pDevminor = strtok__(g_devminor, ";"); \
				pDevname && pDevminor; \
				pDevname = strtok_(NULL, ";"),  pDevminor = strtok__(NULL, ";"))
		{
			uiDevminor = simple_strtol(pDevminor, 0, 10);
			device = (struct class_device *)device_create(motorclass, NULL, MKDEV(g_devmajor, uiDevminor), NULL, pDevname);
			if (IS_ERR(device))
			{
				DBG_ERR("Device creation failure.\n");
				motor_cleanup_module();
				return PTR_ERR(device);
			}
			allocatedTag |= (0x01 << uiDevminor);
			DBG(emDebugMsg, "Register device: %s(%d, %d), %d.\n", pDevname, g_devmajor, uiDevminor, allocatedTag);
		}
	}while(0);

	//hi_ssp_init();

	return S_OK;
}

/*!*************************************************************************
 * Cleanup motor module while rmmod.
 ****************************************************************************/
void __exit motor_cleanup_module ()
{
	INT iIndex;

	for (iIndex = 0; allocatedTag; iIndex++, allocatedTag >>= 1)
	{
		if (allocatedTag & 0x01)
		{
			device_destroy(motorclass, MKDEV(g_devmajor, iIndex));
			DBG(emDebugInfo, "Destroy class device: (%d, %d).\n", g_devmajor, iIndex);
		}
	}

	class_destroy(motorclass);
	unregister_chrdev(g_devmajor, MOTOR_VFS_INTERFACE);

	//hi_ssp_exit();

	printk("\n%s Bye...\n", MOTOR_SYSLOG_ID);
}
/* ============================================================================= */

/* Register driver to the kernel */
module_init(motor_init_module);
module_exit(motor_cleanup_module);
/* ============================================================================= */
MODULE_AUTHOR("Tim.Kuo@vivotek.com");
MODULE_DESCRIPTION("Vivotek Motor Driver");
MODULE_SUPPORTED_DEVICE("Vivotek Motor Driver");
MODULE_LICENSE("GPL");
/* ============================================================================= */
